import java.io.*;
import java.net.*;

public class MirageWriter {
	protected Socket mirageSocket;
	protected PrintWriter out;
	protected String msgName;
	protected String server;

	public MirageWriter(String name, String server) {
		this.server = server;
		try  {
			msgName = name;
			mirageSocket = new Socket(server, 19785);
			out = new PrintWriter(mirageSocket.getOutputStream(), true);
		} catch (UnknownHostException e) {
			System.err.println("Don't know host localhost");
			System.exit(1);
		} catch (IOException e) {
			System.out.println("Couldn't get I/O for localhost:19785");
			System.exit(1);
		}
	}

	public void sendOpeningMessage(JointNode root) {
		String msg = "";
		msg += "<messages>\n";
		msg += "<plist version=\"1.0\"><dict>\n";
		msg += "<key>ID</key> <string>"+msgName+"</string>\n";
		msg += "<key>Location</key> <array>\n<real>0</real>\n<real>0</real>\n<real>0</real>\n</array>\n";
		msg += "<key>Model</key> <array>\n";
		msg += root.toXMLString();
		//msg += "</array></plist>";
		msg += "</dict></plist>\n";
		out.println(msg);
	}

	public void sendUpdateMessage(JointNode root) {
		String msg = "";
		msg += "<plist version=\"1.0\"><dict>\n";
		msg += "<key>ID</key> <string>"+msgName+"</string>\n";
		msg += "<key>Positions</key> <dict>\n";
		msg += getValuesString(root);
		msg += "</dict>\n</dict></plist>";
		out.println(msg);
	}

	public void sendCloseMessage() {
		out.println("</messages>");
	}

	protected String getValuesString(JointNode j) {
		String s = "<key>"+j.name+"</key> <real>";
		if (j.type.equals("revolute")) {
			s += j.theta;
		} else {
			s += j.d;
		}
		s += "</real>";
		for (int i = 0; i < j.children.size(); i++) {
			s += getValuesString((JointNode)(j.children.get(i)));
		}
		return s;
	}

}
