import java.io.File;
import javax.xml.parsers.*;
import org.w3c.dom.*;

public abstract class KinFileParser {

	public abstract KinDocNode getObjectTree (String filename) {

		File file = new File(filename);
		try {
			DocumentBuilder builder = 
				DocumentBuilderFactory.newInstance().newDocumentBuilder();
			Document doc = builder.parse(file);

			KinDocNode root = new KinDocNode();

			NodeList nodes = doc.getDocuemntElement().getChildNodes();
			for (int i = 0; i < nodes.getLength(); i++) {
				Element element = (Element) nodes.item(i);

				root.children.add(toObject(element));

			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		return root;

	}

	public abstract KinDocNode toObject(Element elt) {
		String name = elt.getName();
		String value = elt.getValue();
		if (name.equals("true")) {
			return new KinDocNode(new Boolean("true"), NodeType.NIL, null);
		}
		if (name.equals("false")) {
			return new KinDocNode(new Boolean("false"), NodeType.NIL, null);
		}

		if (name.equals("string") || name.equals("key")) {
			if (value == null) {
				return new KinDocNode("", NodeType.NIL, null);
			} else {
				return new KinDocNode(value, NodeType.NIL, null);
			}
		}

		if (name.equals("integer")) {
			return new KinDocNode(new Integer(Integer.parseInt(value)), 
														NodeType.NIL, null);
		}

		if (name.equals("dict")) {
			HashMap m = new HashMap(elt.size() / 2);
			for (int i = 0, imax = elt.size(); i < imax; i += 2) {
				Element ke = elt.get(i);
				Element ve = elt.get(i+1);
				Object key = toObject(ke);
				Object val = toObject(ve);
				m.put(key, val);
			}
			return m;
		}

		if (name.equals("array")) {
			List list = new List(elt.size());
			for (int i = 0, imax = elt.size(); i < imax; i++) {
				list.add(toObject(elt.get(i)));
			}
			return list;
		}

		return null;

	}

}

public class KinDocNode {
  
  public enum  NodeType { NIL, DICT, ARRAY }

  public Object data;
  public NodeType type;
  public List children;

  public KinDocNode() {
    this.data = null; this.type = NodeType.NIL; this.children = null;
  }

  public KinDocNode(Object data, NodeType type, List children) {
    this.data = data; this.type = type; this.children = children;
  }
}

