import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JSplitPane;
import javax.swing.UIManager;
import javax.swing.event.*;
import javax.swing.*;

import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.tree.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;

import java.io.IOException;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.*;

import java.io.*;
import java.net.*;


public class DHWizard extends JPanel
                      implements TreeSelectionListener, ChangeListener, ActionListener {
    private JPanel infoPanel;
//	private JTextField nameField;
	private JSpinner thetaSpinner, rSpinner, alphaSpinner, dSpinner;
	private JComboBox typeBox;
	private JButton saveButton;
    private JTree tree;
    private static boolean DEBUG = false;
	private static String kinFileName;
	private static String serverName;
	private static boolean connectToServer;
	protected DefaultMutableTreeNode rootNode;
	protected DefaultTreeModel treeModel;
	protected MirageWriter mirageOut;


    //Optionally play with line styles.  Possible values are
    //"Angled" (the default), "Horizontal", and "None".
    private static boolean playWithLineStyle = false;
    private static String lineStyle = "Horizontal";
    
    //Optionally set the look and feel.
    private static boolean useSystemLookAndFeel = false;

    public DHWizard(JointNode node) {
        super(new GridLayout(1,0));

        //Create the nodes.
        rootNode = createTree(node);
		treeModel = new DefaultTreeModel(rootNode);

        //Create a tree that allows one selection at a time.
        tree = new JTree(treeModel);
        tree.getSelectionModel().setSelectionMode
                (TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.setEditable(true);

        //Listen for when the selection changes.
        tree.addTreeSelectionListener(this);

        if (playWithLineStyle) {
            System.out.println("line style = " + lineStyle);
            tree.putClientProperty("JTree.lineStyle", lineStyle);
        }

        //Create the scroll pane and add the tree to it. 
        JScrollPane treeView = new JScrollPane(tree);


        //Create the HTML viewing pane.
        infoPanel = new JPanel(new GridLayout(6,2));

		thetaSpinner = new JSpinner(new SpinnerNumberModel(
								0, 0, 3.1415, 0.01));
		thetaSpinner.addChangeListener(this);
		JLabel thetaLabel = new JLabel("theta");
		infoPanel.add(thetaLabel);
		infoPanel.add(thetaSpinner);

		alphaSpinner = new JSpinner(new SpinnerNumberModel(
								0, 0, 3.1415, 0.01));
		alphaSpinner.addChangeListener(this);
		JLabel alphaLabel = new JLabel("alpha");
		infoPanel.add(alphaLabel);
		infoPanel.add(alphaSpinner);

		dSpinner = new JSpinner(new SpinnerNumberModel(
								0, 0, 3.1415, 0.01));
		dSpinner.addChangeListener(this);
		JLabel dLabel = new JLabel("d");
		infoPanel.add(dLabel);
		infoPanel.add(dSpinner);

		rSpinner = new JSpinner(new SpinnerNumberModel(
								0, 0, 3.1415, 0.01));
		rSpinner.addChangeListener(this);
		JLabel rLabel = new JLabel("r");
		infoPanel.add(rLabel);
		infoPanel.add(rSpinner);

		String [] types = {"revolute", "prismatic"};
		typeBox = new JComboBox (types);
		typeBox.addActionListener(this);
		JLabel typeLabel = new JLabel("type");
		infoPanel.add(typeLabel);
		infoPanel.add(typeBox);

		infoPanel.add(new JLabel(""));

		saveButton = new JButton("save");
		saveButton.setActionCommand("save");
		saveButton.addActionListener(this);
		infoPanel.add(saveButton);


        //JScrollPane infoView = new JScrollPane(infoPane);

        //Add the scroll panes to a split pane.
        JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        splitPane.setLeftComponent(treeView);
        //splitPane.setRightComponent(infoView);
        splitPane.setRightComponent(infoPanel);

        Dimension minimumSize = new Dimension(100, 50);
        //infoView.setMinimumSize(minimumSize);
        infoPanel.setMinimumSize(minimumSize);
        treeView.setMinimumSize(minimumSize);
        splitPane.setDividerLocation(300); 
        splitPane.setPreferredSize(new Dimension(500, 300));

        //Add the split pane to this panel.
        add(splitPane);
			
			if (connectToServer) {
				mirageOut = new MirageWriter("DHwizard", serverName);
				mirageOut.sendOpeningMessage((JointNode)(rootNode.getUserObject()));
			}
    }

    /** Required by TreeSelectionListener interface. */
    public void valueChanged(TreeSelectionEvent e) {
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                           tree.getLastSelectedPathComponent();

        if (node == null) return;

        JointNode joint = (JointNode)node.getUserObject();
		if (DEBUG) {
			System.out.println(joint.name + " selected.");
		}
		displayJointInfo(joint);
    }

	public void stateChanged(ChangeEvent e) {
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                           tree.getLastSelectedPathComponent();

        if (node == null) return;

        JointNode joint = (JointNode)node.getUserObject();
		if (e.getSource() == thetaSpinner) {
			joint.theta = ((Double)thetaSpinner.getValue()).doubleValue();
		} else if (e.getSource() == alphaSpinner) {
			joint.alpha = ((Double)alphaSpinner.getValue()).doubleValue();
		} else if (e.getSource() == rSpinner) {
			joint.r = ((Double)rSpinner.getValue()).doubleValue();
		} else if (e.getSource() == dSpinner) {
			joint.d = ((Double)dSpinner.getValue()).doubleValue();
		}
		else {
			//System.out.println("Source: " + e.getSource());
		}
		if (connectToServer) {
		if ((e.getSource() == thetaSpinner && joint.type.equals("revolute")) ||
		    (e.getSource() == dSpinner && joint.type.equals("prismatic"))) {
			//we can just send a message containing the q values
			mirageOut.sendUpdateMessage((JointNode)(rootNode.getUserObject()));
		}	else {
			//we've edited the structure of the chain
			mirageOut.sendCloseMessage();
			mirageOut.sendOpeningMessage((JointNode)(rootNode.getUserObject()));
		}
		}
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals("save")) {
			try {
				FileWriter f = new FileWriter("DHout.kin");
				BufferedWriter out = new BufferedWriter(f);
				out.write(((JointNode)rootNode.getUserObject()).toXMLString());
				out.close();
			} catch (Exception x) {
			}
		} else if (e.getActionCommand().equals("comboBoxChanged")) {
        	DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                           tree.getLastSelectedPathComponent();

        	if (node == null) return;

        	JointNode joint = (JointNode)node.getUserObject();
			joint.type = (String)typeBox.getSelectedItem();
		}
		else {
			System.out.println(e.getActionCommand());
		}
	}

    private void displayJointInfo(JointNode joint) {
		thetaSpinner.setValue(new Double(joint.theta));
		alphaSpinner.setValue(new Double(joint.alpha));
		dSpinner.setValue(new Double(joint.d));
		rSpinner.setValue(new Double(joint.r));

		if (joint.type.equals("revolute")) {
			((SpinnerNumberModel)thetaSpinner.getModel()).setMaximum(new Double(joint.max));
			((SpinnerNumberModel)thetaSpinner.getModel()).setMinimum(new Double(joint.min));
		}
		else {
			((SpinnerNumberModel)thetaSpinner.getModel()).setMaximum(null);
			((SpinnerNumberModel)thetaSpinner.getModel()).setMinimum(null);
		}

		if (joint.type.equals("prismatic")) {
			((SpinnerNumberModel)dSpinner.getModel()).setMaximum(new Double(joint.max));
			((SpinnerNumberModel)dSpinner.getModel()).setMinimum(new Double(joint.min));
		}
		else {
			((SpinnerNumberModel)dSpinner.getModel()).setMaximum(null);
			((SpinnerNumberModel)dSpinner.getModel()).setMinimum(null);
		}

		typeBox.setSelectedItem(new String(joint.type));
    }

    private DefaultMutableTreeNode createTree(JointNode node) {
        DefaultMutableTreeNode result;
		
		result = new DefaultMutableTreeNode(node);
		int size = node.children.size();
		for (int i = 0; i < size; i++) {
			result.add(createTree((JointNode)node.children.get(i)));
		}

		return result;
    }
        
    /**
     * Create the GUI and show it.  For thread safety,
     * this method should be invoked from the
     * event dispatch thread.
     */
    private static void createAndShowGUI() {
        if (useSystemLookAndFeel) {
            try {
                UIManager.setLookAndFeel(
                    UIManager.getSystemLookAndFeelClassName());
            } catch (Exception e) {
                System.err.println("Couldn't use system look and feel.");
            }
        }

		KinParser k = new KinParser();

		JointNode base = k.parse(kinFileName);

        //Create and set up the window.
        JFrame frame = new JFrame("DHWizard");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        //Add content to the window.
        frame.add(new DHWizard(base));

        //Display the window.
        frame.pack();
        frame.setVisible(true);
    }

    public static void main(String[] args) {
        //Schedule a job for the event dispatch thread:
        //creating and showing this application's GUI.
		kinFileName = args[0];
		if (args.length > 1) {
			serverName = args[1];
			connectToServer = true;
		} else {
			connectToServer = false;
		}
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        });
    }
}
