import java.io.File;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.w3c.dom.traversal.*;


public class KinParser {

	private TreeWalker walker;

	private int i;

	public KinParser() {}

	public JointNode parse (String fname) {
		File file = new File(fname);
		JointNode j = null;
		try {
			DocumentBuilder builder = 
				DocumentBuilderFactory.newInstance().newDocumentBuilder();
			Document document = builder.parse(file);

    		DocumentTraversal traversal = (DocumentTraversal) document;

	    	walker = traversal.createTreeWalker(
				    document.getDocumentElement(), 
					NodeFilter.SHOW_ELEMENT, null, true);

			i = 0;

			j = parse();

		} catch (Exception e) {
			e.printStackTrace();
		}

		return j;
	}

	public JointNode parse() {
		JointNode j = new JointNode();
		Node n = walker.getCurrentNode();
		if (n.getNodeName().equals("array")) {
			walker.firstChild();
			j = parseArray();
			walker.setCurrentNode(n);
		}
		else if (n.getNodeName().equals("dict")) {
			walker.firstChild();
			j = parseDict();
			walker.setCurrentNode(n);
			if (walker.nextSibling() != null) {
				j.addChild(parse());
			}
		}
		else if (n.getNodeName().equals("plist")) {
			walker.firstChild();
			j = parse();
		}
		return j;
	}

	public JointNode parseArray() {
		JointNode j;
		Node n = walker.getCurrentNode();
		j = parse();
		while (walker.nextSibling() != null) {
			j.addChild(parse());
		}
		walker.setCurrentNode(n);
		return j;
	}

	public JointNode parseDict() {
		JointNode j = new JointNode();
		for (Node key = walker.getCurrentNode(); key != null;
				key = walker.nextSibling()) {
			Node val = walker.nextSibling();
			if (key.getFirstChild() != null &&
				key.getFirstChild().getNodeName().equals("#text") &&
				val.getFirstChild() != null &&
				val.getFirstChild().getNodeName().equals("#text")) {
				String s = key.getFirstChild().getNodeValue();
				String t = val.getFirstChild().getNodeValue();
				if (t.matches("^\\S+$") && s.matches("^\\S+$")) {
					updateJoint(j, s, t);	
				}
			}
		}
		return j;
	}

	private void updateJoint(JointNode j, String key, String val) {
		if (key.equals("Name")) {
			j.name = val;
		} else if (key.equals("θ")) {
			if (val.indexOf("°") != -1) {
				j.theta = Math.toRadians(Double.parseDouble(
					val.substring(0,val.indexOf("°"))));
			} else {
				j.theta = Double.parseDouble(val);
			}
		} else if (key.equals("r")) {
			j.r = Double.parseDouble(val);
		} else if (key.equals("α")) {
			if (val.indexOf("°") != -1) {
				j.alpha = Math.toRadians(Double.parseDouble(
					val.substring(0, val.indexOf("°"))));
			} else {
				j.alpha = Double.parseDouble(val);
			}
		} else if (key.equals("d")) {
			j.d = Double.parseDouble(val);
		} else if (key.equals("qOffset")) {
			if (val.indexOf("°") != -1) {
				j.qOffset = Math.toRadians(Double.parseDouble(
					val.substring(0, val.indexOf("°"))));
			} else {
				j.qOffset = Double.parseDouble(val);
			}
		} else if (key.equals("Min")) {
			if (val.indexOf("°") != -1) {
				j.min = Math.toRadians(Double.parseDouble(
					val.substring(0, val.indexOf("°"))));
			} else {
				j.min = Double.parseDouble(val);
			}
		} else if (key.equals("Max")) {
			if (val.indexOf("°") != -1) {
				j.max = Math.toRadians(Double.parseDouble(
					val.substring(0, val.indexOf("°"))));
			} else {
				j.max = Double.parseDouble(val);
			}
		} else if (key.equals("Mass")) {
			j.mass = Double.parseDouble(val);
		} else if (key.equals("Model")) {
			j.model = val;
		} else if (key.equals("IKSolver")) {
			j.iksolver = val;
		} else if (key.equals("Material")) {
			j.material = val;
		} else if (key.equals("JointType")) {
			j.type = val;
		}
	}
}

